@@ -16,13 +16,14 @@
 #include <gconf/gconf-client.h>
 
 #include "ui.h"
+#include "book-view.h"
 
 
 enum {
-    RENDER_PREVIEW,
     START_SCAN,
     STOP_SCAN,
     SAVE,
+    EMAIL,
     PRINT,
     QUIT,
     LAST_SIGNAL
@@ -33,17 +34,23 @@ static guint signals[LAST_SIGNAL] = { 0, };
 struct SimpleScanPrivate
 {
     GConfClient *client;
+    
+    GtkBuilder *builder;
 
     GtkWidget *window;
-    GtkWidget *scan_label;
+    GtkWidget *scan_button_label, *continuous_scan_button_label, *page_label;
     GtkWidget *actions_box;
     GtkWidget *device_combo, *mode_combo;
     GtkTreeModel *device_model, *mode_model;
+    GtkWidget *replace_pages_check;
     GtkWidget *preview_area;
+    GtkWidget *zoom_scale;
+    GtkWidget *page_delete_menuitem, *crop_rotate_menuitem;
+    BookView *book_view;
+    gboolean updating_page_menu;
 
     gchar *default_file_name;
     gboolean scanning;
-    Orientation orientation;
 };
 
 G_DEFINE_TYPE (SimpleScan, ui, G_TYPE_OBJECT);
@@ -68,21 +75,6 @@ find_scan_device (SimpleScan *ui, const char *device, GtkTreeIter *iter)
 }
 
 
-static gchar *
-get_selected_device (SimpleScan *ui)
-{
-    GtkTreeIter iter;
-
-    if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (ui->priv->device_combo), &iter)) {
-        gchar *device;
-        gtk_tree_model_get (ui->priv->device_model, &iter, 0, &device, -1);
-        return device;
-    }
-
-    return NULL;
-}
-
-
 void
 ui_set_default_file_name (SimpleScan *ui, const gchar *default_file_name)
 {
@@ -122,6 +114,21 @@ ui_add_scan_device (SimpleScan *ui, const gchar *device, const gchar *label)
 }
 
 
+gchar *
+ui_get_selected_device (SimpleScan *ui)
+{
+    GtkTreeIter iter;
+
+    if (gtk_combo_box_get_active_iter (GTK_COMBO_BOX (ui->priv->device_combo), &iter)) {
+        gchar *device;
+        gtk_tree_model_get (ui->priv->device_model, &iter, 0, &device, -1);
+        return device;
+    }
+
+    return NULL;
+}
+
+
 void
 ui_set_selected_device (SimpleScan *ui, const gchar *device)
 {
@@ -173,25 +180,14 @@ get_document_hint (SimpleScan *ui)
 }
 
 
-G_MODULE_EXPORT
-gboolean
-preview_area_expose_event_cb (GtkWidget *widget, GdkEventExpose *event, SimpleScan *ui)
+static gboolean
+get_replace_pages (SimpleScan *ui)
 {
-    cairo_t *context;
-    double width, height;
-    
-    context = gdk_cairo_create (widget->window);
-    
-    width = widget->allocation.width;
-    height = widget->allocation.height;
-    g_signal_emit (G_OBJECT (ui), signals[RENDER_PREVIEW], 0, context, width, height);
-
-    cairo_destroy (context);
-
-    return FALSE;
+    return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check));
 }
 
 
+void scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
 scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
@@ -201,10 +197,11 @@ scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
     } else {
         gchar *device, *mode;
 
-        device = get_selected_device (ui);
+        device = ui_get_selected_device (ui);
         if (device) {
             mode = get_document_hint (ui);
-            g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, mode);
+            g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, mode,
+                           FALSE, get_replace_pages (ui));
             g_free (device);
             g_free (mode);
         }
@@ -212,17 +209,221 @@ scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 }
 
 
+void continuous_scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
-rotate_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+continuous_scan_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 {
-    ui->priv->orientation++;
-    if (ui->priv->orientation > RIGHT_TO_LEFT)
-        ui->priv->orientation = TOP_TO_BOTTOM;
-    ui_redraw_preview (ui);
+    if (ui->priv->scanning) {
+        g_signal_emit (G_OBJECT (ui), signals[STOP_SCAN], 0);
+    } else {
+        gchar *device, *mode;
+
+        device = ui_get_selected_device (ui);
+        if (device) {
+            mode = get_document_hint (ui);
+            g_signal_emit (G_OBJECT (ui), signals[START_SCAN], 0, device, mode,
+                           TRUE, get_replace_pages (ui));
+            g_free (device);
+            g_free (mode);
+        }
+    }
 }
 
 
+static void
+page_selected_cb (BookView *view, Page *page, SimpleScan *ui)
+{
+    char *name = NULL;
+
+    if (page == NULL)
+        return;
+
+    ui->priv->updating_page_menu = TRUE;
+    
+    if (page_has_crop (page)) {
+        char *crop_name;
+
+        // FIXME: Make more generic
+        crop_name = page_get_named_crop (page);
+        if (crop_name) {
+            if (strcmp (crop_name, "A4") == 0)
+                name = "a4_menuitem";
+            else if (strcmp (crop_name, "A5") == 0)
+                name = "a5_menuitem";
+            else if (strcmp (crop_name, "A6") == 0)
+                name = "a6_menuitem";
+            else if (strcmp (crop_name, "letter") == 0)
+                name = "letter_menuitem";
+            else if (strcmp (crop_name, "legal") == 0)
+                name = "legal_menuitem";
+            else if (strcmp (crop_name, "4x6") == 0)
+                name = "4x6_menuitem";
+            g_free (crop_name);
+        }
+        else
+            name = "custom_crop_menuitem";
+    }
+    else
+        name = "no_crop_menuitem";
+
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (gtk_builder_get_object (ui->priv->builder, name)), TRUE);
+    
+    ui->priv->updating_page_menu = FALSE;
+}
+
+
+void rotate_left_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+rotate_left_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (ui->priv->updating_page_menu)
+        return;
+    page_rotate_left (book_view_get_selected (ui->priv->book_view));
+}
+
+
+void rotate_right_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+rotate_right_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (ui->priv->updating_page_menu)
+        return;
+    page_rotate_right (book_view_get_selected (ui->priv->book_view));    
+}
+
+
+static void
+set_crop (SimpleScan *ui, const gchar *crop_name)
+{
+    Page *page;
+    
+    gtk_widget_set_sensitive (ui->priv->crop_rotate_menuitem, crop_name != NULL);
+
+    if (ui->priv->updating_page_menu)
+        return;
+    
+    page = book_view_get_selected (ui->priv->book_view);
+    if (!page)
+        return;
+    
+    if (!crop_name) {
+        page_set_no_crop (page);
+        return;
+    }
+    
+    page_set_named_crop (page, crop_name);
+}
+
+
+void no_crop_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+no_crop_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_crop (ui, NULL);
+}
+
+
+void custom_crop_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+custom_crop_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_crop (ui, "custom");
+}
+
+
+void four_by_six_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+four_by_six_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_crop (ui, "4x6");
+}
+
+                         
+void legal_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+legal_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_crop (ui, "legal");
+}
+
+                         
+void letter_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+letter_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_crop (ui, "letter");
+}
+
+                         
+void a6_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+a6_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_crop (ui, "A6");
+}
+
+
+void a5_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+a5_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_crop (ui, "A5");
+}
+
+                         
+void a4_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+a4_menuitem_toggled_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    if (gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget)))
+        set_crop (ui, "A4");
+}
+
+
+void crop_rotate_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+crop_rotate_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    Page *page;
+
+    page = book_view_get_selected (ui->priv->book_view);
+    if (!page)
+        return;
+    
+    page_rotate_crop (page);
+}
+
+
+void page_delete_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+page_delete_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    book_delete_page (book_view_get_book (ui->priv->book_view),
+                      book_view_get_selected (ui->priv->book_view));
+}
+
+
+void save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
 save_file_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
@@ -271,6 +472,22 @@ draw_page (GtkPrintOperation *operation,
 }
 
 
+void email_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+email_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    g_signal_emit (G_OBJECT (ui), signals[EMAIL], 0);
+    GError *error = NULL;
+    g_spawn_command_line_async ("xdg-email", &error);
+    if (error) {
+        g_warning ("Unable to start email: %s", error->message);
+        g_error_free (error);
+    }
+}
+
+
+void print_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
 void
 print_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
@@ -292,6 +509,24 @@ print_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
 }
 
 
+void next_page_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+next_page_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    book_view_select_next_page (ui->priv->book_view);
+}
+
+
+void prev_page_button_clicked_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+prev_page_button_clicked_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    book_view_select_prev_page (ui->priv->book_view);    
+}
+
+
 static void
 load_device_cache (SimpleScan *ui)
 {
@@ -408,15 +643,54 @@ save_device_cache (SimpleScan *ui)
 }
 
 
+void about_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui);
 G_MODULE_EXPORT
-gboolean
-window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui)
+void
+about_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    const gchar *authors[] = { "Robert Ancell <robert.ancell@canonical.com>", NULL };
+
+    /* The license this software is under (GPL3+) */
+    const char *license = _("This program is free software: you can redistribute it and/or modify\n"
+                            "it under the terms of the GNU General Public License as published by\n"
+                            "the Free Software Foundation, either version 3 of the License, or\n"
+                            "(at your option) any later version.\n"
+                            "\n"
+                            "This program is distributed in the hope that it will be useful,\n"
+                            "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+                            "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
+                            "GNU General Public License for more details.\n"
+                            "\n"
+                            "You should have received a copy of the GNU General Public License\n"
+                            "along with this program.  If not, see <http://www.gnu.org/licenses/>.");
+
+    /* Title of about dialog */
+    const char *title = _("About Simple Scan");
+
+    /* Description of program */
+    const char *description = _("Simple document scanning tool");
+
+    gtk_show_about_dialog (GTK_WINDOW (ui->priv->window),
+                           "title", title,
+                           "program-name", "Simple Scan",
+                           "version", VERSION,
+                           "comments", description,
+                           "authors", authors,
+                           "website", "https://launchpad.net/simple-scan",
+                           "copyright", "Copyright (C) 2009 Canonical Ltd.",
+                           "license", license,
+                           NULL);
+}
+
+
+static void
+quit (SimpleScan *ui)
 {
     char *device, *document_type;
 
     save_device_cache (ui);
 
-    device = get_selected_device (ui);
+    device = ui_get_selected_device (ui);
     if (device) {
         gconf_client_set_string(ui->priv->client, "/apps/simple-scan/selected_device", device, NULL);
         g_free (device);
@@ -426,7 +700,28 @@ window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui)
     gconf_client_set_string(ui->priv->client, "/apps/simple-scan/document_type", document_type, NULL);
     g_free (document_type);
 
+    gconf_client_set_bool(ui->priv->client, "/apps/simple-scan/replace_pages",
+                          get_replace_pages (ui), NULL);
+
     g_signal_emit (G_OBJECT (ui), signals[QUIT], 0);
+}
+
+
+void quit_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui);
+G_MODULE_EXPORT
+void
+quit_menuitem_activate_cb (GtkWidget *widget, SimpleScan *ui)
+{
+    quit (ui);
+}
+
+
+gboolean window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui);
+G_MODULE_EXPORT
+gboolean
+window_delete_event_cb (GtkWidget *widget, GdkEvent *event, SimpleScan *ui)
+{
+    quit (ui);
     return TRUE;
 }
 
@@ -438,8 +733,9 @@ ui_load (SimpleScan *ui)
     GError *error = NULL;
     GtkCellRenderer *renderer;
     gchar *device, *document_type;
+    gboolean replace_pages;
 
-    builder = gtk_builder_new ();
+    builder = ui->priv->builder = gtk_builder_new ();
     gtk_builder_add_from_file (builder, UI_DIR "simple-scan.ui", &error);
     if (error) {
         g_critical ("Unable to load UI: %s\n", error->message);
@@ -453,13 +749,19 @@ ui_load (SimpleScan *ui)
     gtk_builder_connect_signals (builder, ui);
 
     ui->priv->window = GTK_WIDGET (gtk_builder_get_object (builder, "simple_scan_window"));
-    ui->priv->scan_label = GTK_WIDGET (gtk_builder_get_object (builder, "scan_label"));
+    ui->priv->scan_button_label = GTK_WIDGET (gtk_builder_get_object (builder, "scan_button_label"));
+    ui->priv->continuous_scan_button_label = GTK_WIDGET (gtk_builder_get_object (builder, "continuous_scan_button_label"));
+    ui->priv->page_label = GTK_WIDGET (gtk_builder_get_object (builder, "page_label"));
     ui->priv->actions_box = GTK_WIDGET (gtk_builder_get_object (builder, "actions_box"));
     ui->priv->device_combo = GTK_WIDGET (gtk_builder_get_object (builder, "device_combo"));
     ui->priv->device_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->device_combo));
     ui->priv->mode_combo = GTK_WIDGET (gtk_builder_get_object (builder, "mode_combo"));
     ui->priv->mode_model = gtk_combo_box_get_model (GTK_COMBO_BOX (ui->priv->mode_combo));
+    ui->priv->replace_pages_check = GTK_WIDGET (gtk_builder_get_object (builder, "replace_pages_check"));
     ui->priv->preview_area = GTK_WIDGET (gtk_builder_get_object (builder, "preview_area"));
+    ui->priv->zoom_scale = GTK_WIDGET (gtk_builder_get_object (builder, "zoom_scale"));
+    ui->priv->page_delete_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "page_delete_menuitem"));
+    ui->priv->crop_rotate_menuitem = GTK_WIDGET (gtk_builder_get_object (builder, "crop_rotate_menuitem"));
 
     renderer = gtk_cell_renderer_text_new();
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (ui->priv->device_combo), renderer, TRUE);
@@ -485,6 +787,22 @@ ui_load (SimpleScan *ui)
         set_document_hint (ui, document_type);
         g_free (document_type);
     }
+
+    replace_pages = gconf_client_get_bool (ui->priv->client, "/apps/simple-scan/replace_pages", &error);
+    if (error)
+        g_error_free (error);
+    else
+        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ui->priv->replace_pages_check), replace_pages);
+    
+    ui->priv->book_view = book_view_new ();
+    g_signal_connect (ui->priv->book_view, "page-selected", G_CALLBACK (page_selected_cb), ui);
+    book_view_set_widget (ui->priv->book_view, ui->priv->preview_area,
+                          GTK_WIDGET (gtk_builder_get_object (builder, "page_menu")));
+    gtk_range_set_adjustment (GTK_RANGE (ui->priv->zoom_scale),
+                              book_view_get_zoom_adjustment (ui->priv->book_view));
+    
+    /* Start with a reasonable size */
+    gtk_window_resize (GTK_WINDOW (ui->priv->window), 600, 400);
 }
 
 
@@ -495,18 +813,49 @@ ui_new ()
 }
 
 
+void
+ui_set_book (SimpleScan *ui, Book *book)
+{
+    book_view_set_book (ui->priv->book_view, book);
+}
+
+
+void
+ui_set_selected_page (SimpleScan *ui, Page *page)
+{
+    book_view_select_page (ui->priv->book_view, page);
+}
+
+
+Page *
+ui_get_selected_page (SimpleScan *ui)
+{
+    return book_view_get_selected (ui->priv->book_view);
+}
+
+
 void
 ui_set_scanning (SimpleScan *ui, gboolean scanning)
 {
     ui->priv->scanning = scanning;
-    if (ui->priv->scanning)
-        gtk_label_set_label (GTK_LABEL (ui->priv->scan_label),
+    if (ui->priv->scanning) {
+        gtk_label_set_label (GTK_LABEL (ui->priv->scan_button_label),
                              /* Label on cancel scan button */
                              _("_Cancel"));
-    else
-        gtk_label_set_label (GTK_LABEL (ui->priv->scan_label),
+        gtk_label_set_label (GTK_LABEL (ui->priv->continuous_scan_button_label),
+                             /* Label on cancel scan button */
+                             _("_Cancel"));
+    }
+    else {
+        gtk_label_set_label (GTK_LABEL (ui->priv->scan_button_label),
                              /* Label on scan button */
                              _("_Scan"));
+        gtk_label_set_label (GTK_LABEL (ui->priv->continuous_scan_button_label),
+                             /* Label on continuous scan button */
+                             _("_Continuous Scan"));
+    }
+
+    gtk_widget_set_sensitive (ui->priv->page_delete_menuitem, !scanning);
 }
 
 
@@ -517,19 +866,6 @@ ui_set_have_scan (SimpleScan *ui, gboolean have_scan)
 }
 
 
-Orientation ui_get_orientation (SimpleScan *ui)
-{
-    return ui->priv->orientation;
-}
-
-
-void
-ui_redraw_preview (SimpleScan *ui)
-{
-    gtk_widget_queue_draw (ui->priv->preview_area);    
-}
-
-
 void
 ui_show_error (SimpleScan *ui, const gchar *error_title, const gchar *error_text)
 {
@@ -551,73 +887,30 @@ ui_show_error (SimpleScan *ui, const gchar *error_title, const gchar *error_text
 void
 ui_start (SimpleScan *ui)
 {
-    if (gtk_tree_model_iter_n_children (ui->priv->device_model, NULL) == 0)
-        ui_show_error (ui,
-                       /* Warning displayed when no scanners are detected */
-                       _("No scanners detected"),
-                       /* Hint to user on why there are no scanners detected */
-                       _("Please check your scanner is connected and powered on"));
+    gtk_widget_show (ui->priv->window);
 }
 
 
 /* Generated with glib-genmarshal */
 static void
-g_cclosure_user_marshal_VOID__POINTER_DOUBLE_DOUBLE (GClosure     *closure,
-                                                     GValue       *return_value G_GNUC_UNUSED,
-                                                     guint         n_param_values,
-                                                     const GValue *param_values,
-                                                     gpointer      invocation_hint G_GNUC_UNUSED,
-                                                     gpointer      marshal_data)
+g_cclosure_user_marshal_VOID__STRING_STRING_BOOLEAN_BOOLEAN (GClosure     *closure,
+                                                             GValue       *return_value G_GNUC_UNUSED,
+                                                             guint         n_param_values,
+                                                             const GValue *param_values,
+                                                             gpointer      invocation_hint G_GNUC_UNUSED,
+                                                             gpointer      marshal_data)
 {
-    typedef void (*GMarshalFunc_VOID__POINTER_DOUBLE_DOUBLE) (gpointer     data1,
-                                                              gpointer     arg_1,
-                                                              gdouble      arg_2,
-                                                              gdouble      arg_3,
-                                                              gpointer     data2);
-    register GMarshalFunc_VOID__POINTER_DOUBLE_DOUBLE callback;
-    register GCClosure *cc = (GCClosure*) closure;
-    register gpointer data1, data2;
-    
-    g_return_if_fail (n_param_values == 4);
-    
-    if (G_CCLOSURE_SWAP_DATA (closure))
-    {
-        data1 = closure->data;
-        data2 = g_value_peek_pointer (param_values + 0);
-    }
-    else
-    {
-        data1 = g_value_peek_pointer (param_values + 0);
-        data2 = closure->data;
-    }
-    callback = (GMarshalFunc_VOID__POINTER_DOUBLE_DOUBLE) (marshal_data ? marshal_data : cc->callback);
-    
-    callback (data1,
-              g_value_get_pointer (param_values + 1),
-              g_value_get_double (param_values + 2),
-              g_value_get_double (param_values + 3),
-              data2);
-}
-
-
-/* Generated with glib-genmarshal */
-void
-g_cclosure_user_marshal_VOID__STRING_STRING (GClosure     *closure,
-                                             GValue       *return_value G_GNUC_UNUSED,
-                                             guint         n_param_values,
-                                             const GValue *param_values,
-                                             gpointer      invocation_hint G_GNUC_UNUSED,
-                                             gpointer      marshal_data)
-{
-  typedef void (*GMarshalFunc_VOID__STRING_STRING) (gpointer       data1,
-                                                    gconstpointer  arg_1,
-                                                    gconstpointer  arg_2,
-                                                    gpointer       data2);
-  register GMarshalFunc_VOID__STRING_STRING callback;
+  typedef void (*GMarshalFunc_VOID__STRING_STRING_BOOLEAN_BOOLEAN) (gpointer       data1,
+                                                                    gconstpointer  arg_1,
+                                                                    gconstpointer  arg_2,
+                                                                    gboolean       arg_3,
+                                                                    gboolean       arg_4,
+                                                                    gpointer       data2);
+  register GMarshalFunc_VOID__STRING_STRING_BOOLEAN_BOOLEAN callback;
   register GCClosure *cc = (GCClosure*) closure;
   register gpointer data1, data2;
 
-  g_return_if_fail (n_param_values == 3);
+  g_return_if_fail (n_param_values == 5);
 
   if (G_CCLOSURE_SWAP_DATA (closure))
     {
@@ -629,11 +922,13 @@ g_cclosure_user_marshal_VOID__STRING_STRING (GClosure     *closure,
       data1 = g_value_peek_pointer (param_values + 0);
       data2 = closure->data;
     }
-  callback = (GMarshalFunc_VOID__STRING_STRING) (marshal_data ? marshal_data : cc->callback);
+  callback = (GMarshalFunc_VOID__STRING_STRING_BOOLEAN_BOOLEAN) (marshal_data ? marshal_data : cc->callback);
 
   callback (data1,
             g_value_get_string (param_values + 1),
             g_value_get_string (param_values + 2),
+            g_value_get_boolean (param_values + 3),
+            g_value_get_boolean (param_values + 4),
             data2);
 }
 
@@ -641,22 +936,14 @@ g_cclosure_user_marshal_VOID__STRING_STRING (GClosure     *closure,
 static void
 ui_class_init (SimpleScanClass *klass)
 {
-    signals[RENDER_PREVIEW] =
-        g_signal_new ("render-preview",
-                      G_TYPE_FROM_CLASS (klass),
-                      G_SIGNAL_RUN_LAST,
-                      G_STRUCT_OFFSET (SimpleScanClass, render_preview),
-                      NULL, NULL,
-                      g_cclosure_user_marshal_VOID__POINTER_DOUBLE_DOUBLE,
-                      G_TYPE_NONE, 3, G_TYPE_POINTER, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
     signals[START_SCAN] =
         g_signal_new ("start-scan",
                       G_TYPE_FROM_CLASS (klass),
                       G_SIGNAL_RUN_LAST,
                       G_STRUCT_OFFSET (SimpleScanClass, start_scan),
                       NULL, NULL,
-                      g_cclosure_user_marshal_VOID__STRING_STRING,
-                      G_TYPE_NONE, 2, G_TYPE_STRING, G_TYPE_STRING);
+                      g_cclosure_user_marshal_VOID__STRING_STRING_BOOLEAN_BOOLEAN,
+                      G_TYPE_NONE, 4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN);
     signals[STOP_SCAN] =
         g_signal_new ("stop-scan",
                       G_TYPE_FROM_CLASS (klass),
@@ -673,6 +960,14 @@ ui_class_init (SimpleScanClass *klass)
                       NULL, NULL,
                       g_cclosure_marshal_VOID__POINTER,
                       G_TYPE_NONE, 1, G_TYPE_POINTER);
+    signals[EMAIL] =
+        g_signal_new ("email",
+                      G_TYPE_FROM_CLASS (klass),
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (SimpleScanClass, email),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0);
     signals[PRINT] =
         g_signal_new ("print",
                       G_TYPE_FROM_CLASS (klass),
@@ -704,7 +999,5 @@ ui_init (SimpleScan *ui)
 
     ui->priv->default_file_name = g_strdup (_("Scanned Document.pdf"));
     ui->priv->scanning = FALSE;
-    ui->priv->orientation = TOP_TO_BOTTOM;
-
     ui_load (ui);
 }
